@@ -1,8 +1,7 @@
 /*
  * PL/Proxy - easy access to partitioned database.
  *
- * Copyright (c) 2006 Sven Suursoho, Skype Technologies OÜ
- * Copyright (c) 2007 Marko Kreen, Skype Technologies OÜ
+ * Copyright (c) 2006-2020 PL/Proxy Authors
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -66,6 +65,7 @@ static const char *cluster_config_options[] = {
 	"connection_lifetime",
 	"query_timeout",
 	"disable_binary",
+	"modular_mapping",
 	/* deprecated */
 	"keepalive_idle",
 	"keepalive_interval",
@@ -80,8 +80,10 @@ PG_FUNCTION_INFO_V1(plproxy_fdw_validator);
  * Connection count should be non-zero and power of 2.
  */
 static bool
-check_valid_partcount(int n)
+check_valid_partcount(int n, int modular_mapping)
 {
+	if (modular_mapping)
+		return n > 0;
 	return (n > 0) && !(n & (n - 1));
 }
 
@@ -249,6 +251,11 @@ add_connection(ProxyCluster *cluster, const char *connstr, int part_num)
 
 		aatree_insert(&cluster->conn_tree, (uintptr_t)connstr, &conn->node);
 	}
+	if (cluster->part_map[part_num])
+		ereport(ERROR,
+			(errcode(ERRCODE_SYNTAX_ERROR),
+			 errmsg("Pl/Proxy: duplicate partition in config: %d", part_num),
+			 errhint("already got number %d", part_num)));
 
 	cluster->part_map[part_num] = conn;
 }
@@ -303,6 +310,8 @@ set_config_key(ProxyFunction *func, ProxyConfig *cf, const char *key, const char
 		cf->query_timeout = atoi(val);
 	else if (pg_strcasecmp("disable_binary", key) == 0)
 		cf->disable_binary = atoi(val);
+	else if (pg_strcasecmp("modular_mapping", key) == 0)
+		cf->modular_mapping = atoi(val);
 	else if (pg_strcasecmp("keepalive_idle", key) == 0
 		|| pg_strcasecmp("keepalive_interval", key) == 0
 		|| pg_strcasecmp("keepalive_count", key) == 0)
@@ -399,7 +408,7 @@ reload_parts(ProxyCluster *cluster, Datum dname, ProxyFunction *func)
 	err = SPI_execute_plan(partlist_plan, &dname, NULL, false, 0);
 	if (err != SPI_OK_SELECT)
 		plproxy_error(func, "get_partlist: spi error");
-	if (!check_valid_partcount(SPI_processed))
+	if (!check_valid_partcount(SPI_processed, cluster->config.modular_mapping))
 		plproxy_error(func, "get_partlist: invalid part count");
 
 	/* check column types */
@@ -480,7 +489,9 @@ plproxy_fdw_validator(PG_FUNCTION_ARGS)
 	List	   *options_list = untransformRelOptions(PG_GETARG_DATUM(0));
 	Oid			catalog = PG_GETARG_OID(1);
 	ListCell   *cell;
-	int			part_count = 0;
+	int			part_count = 0, part_num;
+	unsigned char *part_set = NULL;
+	int			modular_mapping = 0;
 
 	/* Pre 8.4.3 databases have broken validator interface, warn the user */
 	if (catalog == InvalidOid)
@@ -497,23 +508,33 @@ plproxy_fdw_validator(PG_FUNCTION_ARGS)
 	{
 		DefElem    *def = lfirst(cell);
 		char	   *arg = strVal(def->arg);
-		int			part_num;
 
 		if (catalog == ForeignServerRelationId)
 		{
 			if (extract_part_num(def->defname, &part_num))
 			{
-				/* partition definition */
-				if (part_num != part_count)
-					ereport(ERROR,
-							(errcode(ERRCODE_SYNTAX_ERROR),
-							 errmsg("Pl/Proxy: partitions must be numbered consecutively"),
-							 errhint("next valid partition number is %d", part_count)));
+				if (part_set == NULL)
+						part_set = palloc0(options_list->length + 1);
+
+				if (part_num < 0 || part_num >= options_list->length)
+						ereport(ERROR,
+								(errcode(ERRCODE_SYNTAX_ERROR),
+								 errmsg("Pl/Proxy: partition numbers must start from 0 and be numbered consecutively"),
+								 errhint("number of options is %d, got %d",
+										 options_list->length, part_num)));
+				if (part_set[part_num])
+						ereport(ERROR,
+								(errcode(ERRCODE_SYNTAX_ERROR),
+								 errmsg("Pl/Proxy: duplicate partition number: %d", part_num),
+								 errhint("got %d twice", part_num)));
+				part_set[part_num] = 1;
 				++part_count;
 			}
 			else
 			{
 				validate_cluster_option(def->defname, arg);
+				if (pg_strcasecmp(def->defname, "modular_mapping") == 0)
+					modular_mapping = atoi(arg);
 			}
 		}
 		else if (catalog == UserMappingRelationId)
@@ -536,16 +557,53 @@ plproxy_fdw_validator(PG_FUNCTION_ARGS)
 
 	if (catalog == ForeignServerRelationId)
 	{
-		if (!check_valid_partcount(part_count))
+		for (part_num = 0; part_num < part_count; part_num++) {
+			if (!part_set[part_num])
+				ereport(ERROR,
+						(errcode(ERRCODE_SYNTAX_ERROR),
+						 errmsg("Pl/Proxy: missing partition"),
+						 errhint("missing number: %d", part_num)));
+		}
+		if (!check_valid_partcount(part_count, modular_mapping))
 			ereport(ERROR,
 					(errcode(ERRCODE_SYNTAX_ERROR),
 					 errmsg("Pl/Proxy: invalid number of partitions"),
 					 errhint("the number of partitions in a cluster must be power of 2 (attempted %d)", part_count)));
+
+		foreach(cell, options_list)
+		{
+			DefElem    *def = lfirst(cell);
+
+			if (!extract_part_num(def->defname, &part_num))
+				continue;
+
+			if (part_num < 0 || part_num >= part_count)
+				ereport(ERROR,
+						(errcode(ERRCODE_SYNTAX_ERROR),
+						 errmsg("Pl/Proxy: wrong partitions number - %d", part_num),
+						 errhint("the partitions number in a cluster must be >= 0 and < %d (attempted %d)", part_count, part_num)));
+		}
 	}
 
+	if (part_set)
+		pfree(part_set);
+
 	PG_RETURN_BOOL(true);
 }
 
+void
+plproxy_append_cstr_option(StringInfo cstr, const char *name, const char *val)
+{
+	/* apply libpq connect string quoting */
+	appendStringInfo(cstr, " %s='", name);
+	for (; *val; val++) {
+		if (*val == '\'' || *val == '\\')
+			appendStringInfoChar(cstr, '\\');
+		appendStringInfoChar(cstr, *val);
+	}
+	appendStringInfoChar(cstr, '\'');
+}
+
 static void
 reload_sqlmed_user(ProxyFunction *func, ProxyCluster *cluster)
 {
@@ -604,12 +662,12 @@ reload_sqlmed_user(ProxyFunction *func, ProxyCluster *cluster)
 		if (strcmp(def->defname, "user") == 0)
 			got_user = true;
 
-		appendStringInfo(&cstr, " %s='%s'", def->defname, strVal(def->arg));
+		plproxy_append_cstr_option(&cstr, def->defname, strVal(def->arg));
 	}
 
 	/* make sure we have 'user=' in connect string */
 	if (!got_user)
-		appendStringInfo(&cstr, " user='%s'", userinfo->username);
+		plproxy_append_cstr_option(&cstr, "user", userinfo->username);
 
 	/* free old string */
 	if (userinfo->extra_connstr)
@@ -639,6 +697,8 @@ reload_sqlmed_cluster(ProxyFunction *func, ProxyCluster *cluster,
 	ListCell		   *cell;
 	int					part_count = 0;
 	int					part_num;
+	int					i;
+	char			  **part_ordered;
 
 
 	fdw = GetForeignDataWrapper(foreign_server->fdwid);
@@ -687,16 +747,13 @@ reload_sqlmed_cluster(ProxyFunction *func, ProxyCluster *cluster,
 
 		if (extract_part_num(def->defname, &part_num))
 		{
-			if (part_num != part_count)
-				plproxy_error(func, "partitions numbers must be consecutive");
-
 			part_count++;
 		}
 		else
 			set_config_key(func, &cluster->config, def->defname, strVal(def->arg));
 	}
 
-	if (!check_valid_partcount(part_count))
+	if (!check_valid_partcount(part_count, cluster->config.modular_mapping))
 		plproxy_error(func, "invalid partition count");
 
 	/*
@@ -705,15 +762,31 @@ reload_sqlmed_cluster(ProxyFunction *func, ProxyCluster *cluster,
 	 */
 	allocate_cluster_partitions(cluster, part_count);
 
+	part_ordered = palloc0(part_count * sizeof(char *));
+
 	foreach(cell, foreign_server->options)
 	{
 		DefElem    *def = lfirst(cell);
+		char	   *arg = strVal(def->arg);
 
 		if (!extract_part_num(def->defname, &part_num))
 			continue;
 
-		add_connection(cluster, strVal(def->arg), part_num);
+		if (part_num < 0 || part_num >= part_count)
+			plproxy_error(func, "wrong partitions number, must be >= 0 and < %d", part_count);
+
+		if (part_ordered[part_num])
+			plproxy_error(func, "duplicate partition number: %d", part_num);
+
+		part_ordered[part_num] = arg;
 	}
+
+	for(i = 0; i < part_count; i++)
+	{
+		add_connection(cluster, part_ordered[i], i);
+	}
+
+	pfree(part_ordered);
 }
 
 /*
@@ -861,8 +934,8 @@ reload_plproxy_cluster(ProxyFunction *func, ProxyCluster *cluster)
 	/* update if needed */
 	if (cur_version != cluster->version || cluster->needs_reload)
 	{
-		reload_parts(cluster, dname, func);
 		get_config(cluster, dname, func);
+		reload_parts(cluster, dname, func);
 		cluster->version = cur_version;
 	}
 }
@@ -1062,8 +1135,8 @@ fake_cluster(ProxyFunction *func, const char *connect_str)
 	cluster->version = 1;
 	cluster->part_count = 1;
 	cluster->part_mask = 0;
-	cluster->part_map = palloc(cluster->part_count * sizeof(ProxyConnection *));
-	cluster->active_list = palloc(cluster->part_count * sizeof(ProxyConnection *));
+	cluster->part_map = palloc0(cluster->part_count * sizeof(ProxyConnection *));
+	cluster->active_list = palloc0(cluster->part_count * sizeof(ProxyConnection *));
 
 	MemoryContextSwitchTo(old_ctx);
 
