@@ -1,8 +1,7 @@
 /*
  * PL/Proxy - easy access to partitioned database.
  *
- * Copyright (c) 2006 Sven Suursoho, Skype Technologies OÜ
- * Copyright (c) 2007 Marko Kreen, Skype Technologies OÜ
+ * Copyright (c) 2006-2020 PL/Proxy Authors
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -29,7 +28,14 @@
 
 #include <sys/time.h>
 
-#include "poll_compat.h"
+/* find poll() */
+#if defined(HAVE_POLL_H)
+#include <poll.h>
+#elif defined(WIN32)
+#define poll(fds, nfds, timeout_ms) WSAPoll(fds, nfds, timeout_ms)
+#elif !defined(POLLIN)
+#error "PL/Proxy requires poll() API"
+#endif
 
 /* some error happened */
 static void
@@ -263,10 +269,11 @@ get_connstr(ProxyConnection *conn)
 		return pstrdup(conn->connstr);
 
 	initStringInfo(&cstr);
+	appendStringInfoString(&cstr, conn->connstr);
 	if (info->extra_connstr)
-		appendStringInfo(&cstr, "%s %s", conn->connstr, info->extra_connstr);
+		appendStringInfo(&cstr, " %s", info->extra_connstr);
 	else
-		appendStringInfo(&cstr, "%s user='%s'", conn->connstr, info->username);
+		plproxy_append_cstr_option(&cstr, "user", info->username);
 	return cstr.data;
 }
 
@@ -286,10 +293,11 @@ prepare_conn(ProxyFunction *func, ProxyConnection *conn)
 	{
 		case C_DONE:
 			conn->cur->state = C_READY;
+			/* fallthrough */
 		case C_READY:
 			if (check_old_conn(func, conn, &now))
 				return;
-
+			/* fallthrough */
 		case C_CONNECT_READ:
 		case C_CONNECT_WRITE:
 		case C_QUERY_READ:
@@ -768,9 +776,21 @@ remote_cancel(ProxyFunction *func)
  * Tag & move tagged connections to active list
  */
 
-static void tag_part(struct ProxyCluster *cluster, int i, int tag)
+static void tag_part(struct ProxyCluster *cluster, int64 hash, int tag)
 {
-	ProxyConnection *conn = cluster->part_map[i];
+	ProxyConnection *conn;
+	int64 idx;
+
+	/* map hash to connection index */
+	if (cluster->config.modular_mapping) {
+		if (hash < 0)
+			idx = -(hash % cluster->part_count);
+		else
+			idx = hash % cluster->part_count;
+	} else {
+		idx = hash & cluster->part_mask;
+	}
+	conn = cluster->part_map[idx];
 
 	if (!conn->run_tag)
 		plproxy_activate_connection(conn);
@@ -779,7 +799,7 @@ static void tag_part(struct ProxyCluster *cluster, int i, int tag)
 }
 
 /*
- * Run hash function and tag connections. If any of the hash function 
+ * Run hash function and tag connections. If any of the hash function
  * arguments are mentioned in the split_arrays an element of the array
  * is used instead of the actual array.
  */
@@ -803,7 +823,7 @@ tag_hash_partitions(ProxyFunction *func, FunctionCallInfo fcinfo, int tag,
 	for (i = 0; i < SPI_processed; i++)
 	{
 		bool		isnull;
-		uint32		hashval = 0;
+		int64		hashval = 0;
 		HeapTuple	row = SPI_tuptable->vals[i];
 		Datum		val = SPI_getbinval(row, desc, 1, &isnull);
 
@@ -819,7 +839,6 @@ tag_hash_partitions(ProxyFunction *func, FunctionCallInfo fcinfo, int tag,
 		else
 			plproxy_error(func, "Hash result must be int2, int4 or int8");
 
-		hashval &= cluster->part_mask;
 		tag_part(cluster, hashval, tag);
 	}
 
@@ -880,8 +899,7 @@ tag_run_on_partitions(ProxyFunction *func, FunctionCallInfo fcinfo, int tag,
 			tag_part(cluster, i, tag);
 			break;
 		case R_ANY:
-			i = random() & cluster->part_mask;
-			tag_part(cluster, i, tag);
+			tag_part(cluster, random(), tag);
 			break;
 		default:
 			plproxy_error(func, "uninitialized run_type");
@@ -889,7 +907,7 @@ tag_run_on_partitions(ProxyFunction *func, FunctionCallInfo fcinfo, int tag,
 }
 
 /*
- * Tag the partitions to be run on, if split is requested prepare the 
+ * Tag the partitions to be run on, if split is requested prepare the
  * per-partition split array parameters.
  *
  * This is done by looping over all of the split arrays side-by-side, for each
@@ -1162,4 +1180,3 @@ plproxy_exec(ProxyFunction *func, FunctionCallInfo fcinfo)
 	PG_END_TRY();
 }
 
-
