@@ -1,8 +1,7 @@
 /*
  * PL/Proxy - easy access to partitioned database.
  *
- * Copyright (c) 2006 Sven Suursoho, Skype Technologies OÜ
- * Copyright (c) 2007 Marko Kreen, Skype Technologies OÜ
+ * Copyright (c) 2006-2020 PL/Proxy Authors
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -36,6 +35,7 @@
 #include <catalog/pg_foreign_server.h>
 #include <catalog/pg_user_mapping.h>
 
+#include <access/hash.h>
 #include <access/htup_details.h>
 #include <access/reloptions.h>
 #include <access/tupdesc.h>
@@ -51,7 +51,7 @@
 #include <utils/array.h>
 #include <utils/builtins.h>
 #include <utils/hsearch.h>
-#include "utils/inval.h"
+#include <utils/inval.h>
 #include <utils/lsyscache.h>
 #include <utils/memutils.h>
 #include <utils/syscache.h>
@@ -73,69 +73,6 @@
 #define container_of(ptr, type, field) ((type *)((char *)(ptr) - offsetof(type, field)))
 #endif
 
-/*
- * backwards compat with 8.2
- */
-#ifndef VARDATA_ANY
-#define VARDATA_ANY(x) VARDATA(x)
-#endif
-#ifndef VARSIZE_ANY_EXHDR
-#define VARSIZE_ANY_EXHDR(x) (VARSIZE(x) - VARHDRSZ)
-#endif
-#ifndef PG_DETOAST_DATUM_PACKED
-#define PG_DETOAST_DATUM_PACKED(x) PG_DETOAST_DATUM(x)
-#endif
-
-#ifndef PG_PRINTF_ATTRIBUTE
-#ifdef WIN32
-#define PG_PRINTF_ATTRIBUTE gnu_printf
-#else
-#define PG_PRINTF_ATTRIBUTE printf
-#endif
-#endif
-
-/*
- * backwards compat with 8.4
- */
-#ifndef PROARGMODE_IN
-#define PROARGMODE_IN       'i'
-#endif
-#ifndef PROARGMODE_OUT
-#define PROARGMODE_OUT      'o'
-#endif
-#ifndef PROARGMODE_INOUT
-#define PROARGMODE_INOUT    'b'
-#endif
-#ifndef PROARGMODE_VARIADIC
-#define PROARGMODE_VARIADIC 'v'
-#endif
-#ifndef PROARGMODE_TABLE
-#define PROARGMODE_TABLE    't'
-#endif
-
-#ifndef TYPTYPE_BASE
-#define TYPTYPE_BASE 'b'
-#endif
-#ifndef TYPTYPE_COMPOSITE
-#define TYPTYPE_COMPOSITE 'c'
-#endif
-#ifndef TYPTYPE_DOMAIN
-#define TYPTYPE_DOMAIN 'd'
-#endif
-#ifndef TYPTYPE_ENUM
-#define TYPTYPE_ENUM 'e'
-#endif
-#ifndef TYPTYPE_PSEUDO
-#define TYPTYPE_PSEUDO 'p'
-#endif
-#ifndef TYPTYPE_RANGE
-#define TYPTYPE_RANGE 'r'
-#endif
-
-#ifndef TupleDescAttr
-#define TupleDescAttr(tupdesc, i) ((tupdesc)->attrs[(i)])
-#endif
-
 /*
  * backwards compatibility with v10.
  */
@@ -189,6 +126,7 @@ typedef struct ProxyConfig
 	int			query_timeout;			/* How long query may take (secs) */
 	int			connection_lifetime;	/* How long the connection may live (secs) */
 	int			disable_binary;			/* Avoid binary I/O */
+	int			modular_mapping;		/* Use modulus (%) instead masking (&) */
 	char		default_user[NAMEDATALEN];
 } ProxyConfig;
 
@@ -486,6 +424,7 @@ void		plproxy_syscache_callback_init(void);
 ProxyCluster *plproxy_find_cluster(ProxyFunction *func, FunctionCallInfo fcinfo);
 void		plproxy_cluster_maint(struct timeval * now);
 void		plproxy_activate_connection(struct ProxyConnection *conn);
+void		plproxy_append_cstr_option(StringInfo cstr, const char *name, const char *val);
 
 /* result.c */
 Datum		plproxy_result(ProxyFunction *func, FunctionCallInfo fcinfo);
