@@ -17,16 +17,15 @@ set client_encoding = 'utf8';
 set client_min_messages = 'warning';
 drop database if exists test_enc_proxy;
 drop database if exists test_enc_part;
-create database test_enc_proxy with encoding 'euc_jp' template template0;
+create database test_enc_proxy with encoding 'euc_jp' lc_collate 'C' lc_ctype 'C' template template0;
 create database test_enc_part with encoding 'utf-8' template template0;
 
 -- initialize proxy db
 \c test_enc_proxy
 set client_encoding = 'utf-8';
-create or replace language plpgsql;
 set client_min_messages = 'warning';
 \set ECHO none
-\i sql/plproxy.sql
+create extension plproxy;
 \set ECHO all
 create schema plproxy;
 create or replace function plproxy.get_cluster_version(cluster_name text)
@@ -51,7 +50,6 @@ create function test_encoding3(text) returns setof intl_data as $$
 $$ language plproxy;
 -- initialize part db
 \c test_enc_part
-create or replace language plpgsql;
 set client_min_messages = 'warning';
 set client_encoding = 'utf8';
 create table intl_data (id int4, "コラム" text);
@@ -88,14 +86,13 @@ set client_min_messages = 'warning';
 drop database if exists test_enc_proxy;
 drop database if exists test_enc_part;
 create database test_enc_proxy with encoding 'utf-8' template template0;
-create database test_enc_part with encoding 'euc_jp' template template0;
+create database test_enc_part with encoding 'euc_jp' lc_collate 'C' lc_ctype 'C' template template0;
 
 -- initialize proxy db
 \c test_enc_proxy
-create or replace language plpgsql;
 set client_min_messages = 'warning';
 \set ECHO none
-\i sql/plproxy.sql
+create extension plproxy;
 \set ECHO all
 set client_encoding = 'utf8';
 create schema plproxy;
@@ -122,7 +119,6 @@ $$ language plproxy;
 
 -- initialize part db
 \c test_enc_part
-create or replace language plpgsql;
 set client_min_messages = 'warning';
 set client_encoding = 'utf8';
 create table intl_data (id int4, "コラム" text);
