@@ -3,9 +3,8 @@
 
 set client_min_messages = 'warning';
 
-\i sql/plproxy.sql
+create extension plproxy;
 
-create or replace language plpgsql;
 set client_min_messages = 'warning';
 
 -- create cluster info functions
@@ -61,13 +60,3 @@ create database test_part3;
 drop database if exists test_enc_proxy;
 drop database if exists test_enc_part;
 
-\c test_part
-create or replace language plpgsql;
-\c test_part0
-create or replace language plpgsql;
-\c test_part1
-create or replace language plpgsql;
-\c test_part2
-create or replace language plpgsql;
-\c test_part3
-create or replace language plpgsql;
